import * as AST from "./ast";
import Error, { isError } from "./error";
import Token, { TokenKind } from "./token";

export default class Parser {
  public tokens: Token[];
  public position: number;

  constructor(tokens: Token[]) {
    this.tokens = tokens;
    this.position = 0;
  }

  public parse(): AST.Statement[] | Error {
    const tree: AST.Statement[] = [];

    while (!this.atEnd()) {
      const stmt = this.statement();
      if (isError(stmt)) {
        return stmt;
      } else {
        tree.push(stmt);
      }
    }

    return tree;
  }

  private statement(): AST.Statement | Error {
    const token = this.currentToken();

    switch (token.kind) {
      case TokenKind.SELECT:
        return this.select();
    }

    return new Error(`Unexpected token ${token.kind}`, token.line);
  }

  private select(): AST.Statement | Error {
    this.eat(TokenKind.SELECT);
    const args = this.args();
    if (isError(args)) {
      return args;
    }
    return new AST.Statement({
      arguments: args,
      type: AST.StatementType.SELECT,
    });
  }

  private args(): AST.Number[] | Error {
    const args = [];

    while (true) {
      const arg = this.number();
      if (isError(arg)) {
        return arg;
      }
      args.push(arg);

      if (this.currentToken().kind === TokenKind.COMMA) {
        this.eat(TokenKind.COMMA);
      } else {
        break;
      }
    }

    return args;
  }

  private number(): AST.Number | Error {
    const n = this.eat(TokenKind.NUMBER);
    if (isError(n)) {
      return n;
    }
    return new AST.Number(parseFloat(n.value || ""));
  }

  private eat(kind: TokenKind) {
    const token = this.currentToken();
    if (token.kind === kind) {
      this.advance();
      return token;
    }

    return new Error(`Unexpected token ${token}`, token.line);
  }

  private currentToken(): Token {
    return this.tokens[this.position];
  }

  private advance() {
    this.position += 1;
  }

  private atEnd(): boolean {
    return this.currentToken().kind === TokenKind.EOF;
  }
}
